import os
from os import listdir
from os.path import exists, isfile, join, islink, isdir
import requests
import re


request_data = {
    "project": "testproject",
    "comments": "testcomment",
}

ty = "/change_hook/base"
path =".buildbot"
dockerfile_extra_contents = {}
dockerfile_extra_contents['focal'] = """

# Buildbot
RUN apt-get install -yq --no-install-suggests --no-install-recommends \
    buildbot-worker git subversion python3-dev libffi-dev python3-setuptools \
    python3-pip dumb-init curl openssh-client wget

# buildbot entrypoint
RUN wget -O /usr/local/bin/buildbot_entrypoint.sh https://git.bitmessage.org/Bitmessage/buildbot-scripts/raw/branch/master/docker/bionic/entrypoint.sh
RUN chmod +x /usr/local/bin/buildbot_entrypoint.sh

RUN echo 'buildbot ALL=(ALL) NOPASSWD:ALL' >> /etc/sudoers

USER buildbot

ENTRYPOINT /usr/local/bin/buildbot_entrypoint.sh "$BUILDMASTER" "$WORKERNAME" "$WORKERPASS"

"""

dockerfile_extra_contents['bionic'] = """

# Buildbot
RUN apt-get install -yq --no-install-suggests --no-install-recommends \
    buildbot-slave git subversion python3-dev libffi-dev python3-setuptools \
    python3-pip dumb-init curl openssh-client wget

# buildbot entrypoint
RUN wget -O /usr/local/bin/buildbot_entrypoint.sh https://git.bitmessage.org/Bitmessage/buildbot-scripts/raw/branch/master/docker/bionic/entrypoint.sh
RUN chmod +x /usr/local/bin/buildbot_entrypoint.sh

RUN echo 'buildbot ALL=(ALL) NOPASSWD:ALL' >> /etc/sudoers

USER buildbot

ENTRYPOINT /usr/local/bin/buildbot_entrypoint.sh "$BUILDMASTER" "$WORKERNAME" "$WORKERPASS"

"""

def get_secret():
    with open(path.join(os.environ['HOME'], "multibuild_parent_key.key"),'r') as f:
        data = f.read()
    return data

def list_jobs(directory=".buildbot"):
    """
    list jobs found in a directory
    """
    results = []
    files = ["Dockerfile", "build.sh", "test.sh"]
    for item in listdir(directory):
        flag = False
        for file in files:
            filepath = join(directory, item, file)
            if islink(filepath) or not isfile(filepath):
                flag = True
                break
        if flag:
            continue    
        if (exists(join(directory, item, 'Dockerfile')) and exists(join(directory, item, 'build.sh'))) or exists(join(directory, item, 'test.sh')):
            results.append(item)
    return results


def _get_dockerfile_contents(jobname):
    """
    Read contents of a Dockerfile and add extra contents for the given os_codename
    """
    os_codename='bionic'
    res = ""
    with open(join(path + jobname), "r") as file:
        contents = file.read()
        # accept any line containing FROM and RUN keywords
        res = ""
        inside_allowed_command = False
        for line in contents:
            if re.match(r"(?m)^(FROM|RUN|ENV).*$", line):
                inside_allowed_command = True
            if inside_allowed_command:
                res += line
                l = line.strip()
                if not l.endswith("\\"):
                    inside_allowed_command = False

    return res + dockerfile_extra_contents[os_codename]


def trigger_child_hooks(buildbotUrl: str, repository, branch, directory=".buildbot"):
    request_url = buildbotUrl + ty
    # List all jobs in the directory
    jobs = list_jobs(directory)
    request_headers = {
        "Content-Type": "application/x-www-form-urlencoded",
        "X-Multibuild-Trigger": get_secret(),
        "Accept": "text/plain",
    }

    # Check if build.sh or test.sh exists in each of the jobs
    for job in jobs:
        build_script_exists = False
        test_script_exists = False
        if exists(join(directory, job, "build.sh")):
            build_script_exists = True
        if exists(join(directory, job, "test.sh")):
            test_script_exists = True

        # make a post request
        request_data["properties"] = {
            "dockerfile": _get_dockerfile_contents(
                join(directory, job, "Dockerfile")
            ),
            "build_script_available": is_build_script_available(build_script_exists),
            "test_script_available": is_test_script_available(test_script_exists),
            "repository": repository,
            "branch": branch,
            "jobname": job,
        }

        requests.post(request_url, headers=request_headers, data=request_data)
